# build ids are not currently generated:
# https://code.google.com/p/go/issues/detail?id=5238
#
# also, debuginfo extraction currently fails with
# "Failed to write file: invalid section alignment"
%global debug_package %{nil}

%global scl oracle-armtoolset-1
%global _scl_prefix /opt/oracle
%global _datadir %{_scl_root}/usr/share
%global _rpmconfigdir2 %{_scl_root}/usr/lib/rpm
# based on el7-u5 golang.spec, where %{name} is expanded to golang
# Here, while name is expanded to oracle-armtoolset-N-golang, we still need golang 
# at multiple points
%global go_name golang
%global update_alt_pack_name go

%{?scl:%scl_package golang}

# we are shipping the full contents of src in the data subpackage, which
# contains binary-like things (ELF data for tests, etc)
%global _binaries_in_noarch_packages_terminate_build 0

# Do not check any files in doc or src for requires
%global __requires_exclude_from ^(%{_datadir}|%{_scl_root}/usr/lib)/%{go_name}/(doc|src)/.*$

# Don't alter timestamps of especially the .a files (or else go will rebuild later)
# Actually, don't strip at all since we are not even building debug packages and this corrupts the dwarf testdata
%global __strip /bin/true

# alternatives should install go inside scl enabled environment
%define alternatives_cmd %{!?scl:%{_sbindir}}%{?scl:%{_root_sbindir}}/alternatives
%define alternatives_cmdline %{alternatives_cmd}%{?scl: --altdir %{_sysconfdir}/alternatives --admindir %{_scl_root}/var/lib/alternatives }

# rpmbuild magic to keep from having meta dependency on libc.so.6
%define _use_internal_dependency_generator 0
%define __find_requires %{nil}
%global __spec_install_post /usr/lib/rpm/check-rpaths   /usr/lib/rpm/check-buildroot  \
  /usr/lib/rpm/brp-compress

# Define GOROOT macros
%global goroot          %{_scl_root}/usr/lib/%{go_name}
%global gopath          %{_datadir}/gocode
%global golang_arches   x86_64 aarch64 ppc64le s390x

# Golang build options.

# Buid golang using external/internal(close to cgo disabled) linking.
%ifarch %{golang_arches}
%global external_linker 1
%else
%global external_linker 0
%endif

# Build golang with cgo enabled/disabled(later equals more or less to internal linking).
%ifarch %{golang_arches}
%global cgo_enabled 1
%else
%global cgo_enabled 0
%endif

# Use golang/gcc-go as bootstrap compiler
%ifarch %{golang_arches}
%global golang_bootstrap 1
%else
%global golang_bootstrap 0
%endif

# Controls what ever we fails on failed tests
%ifarch %{golang_arches}
%global fail_on_tests 1
%else
%global fail_on_tests 0
%endif

# Build golang shared objects for stdlib
%ifarch 0
%global shared 1
%else
%global shared 0
%endif

%ifarch x86_64
%global gohostarch  amd64
%endif

%ifarch aarch64
%global gohostarch  arm64
%endif

%ifarch ppc64le
%global gohostarch ppc64le
%endif

%ifarch s390x
%global gohostarch s390x
%endif

%global go_api 1.10
%global go_version 1.10.3

Name:           %{?scl_prefix}golang
Version:        1.10.3
Release:        2%{?dist}
Summary:        The Go Programming Language
# source tree includes several copies of Mark.Twain-Tom.Sawyer.txt under Public Domain
License:        BSD and Public Domain
URL:            http://golang.org/
Source0:        https://storage.googleapis.com/golang/go%{go_version}.src.tar.gz

# The compiler is written in Go. Needs go(1.4+) compiler for build.
%if !%{golang_bootstrap}
BuildRequires:  gcc-go >= 5
%else
BuildRequires:  golang >= 1.4
%endif
%if 0%{?rhel} > 6 || 0%{?fedora} > 0
BuildRequires:  hostname
%else
BuildRequires:  net-tools
%endif
# for tests
BuildRequires:  pcre-devel, glibc-static, perl

%{?scl:Provides:go = %{version}-%{release}}
%{?scl:Provides:go-srpm-macros = %{version}-%{release}}
Requires: %{scl}
Requires:       %{name}-bin = %{version}-%{release}
Requires:       %{name}-src = %{version}-%{release}

# https://github.com/golang/go/commit/da769814b83c4eb65681dbe682d07d4b902b554a
Patch220:       ./cgo-int-overflow.patch
# disable broken test cases addressed in Orabug 28829875 and go issue 28334
Patch221:       ./aarch64-testshared-shared-test-disable.patch

# Having documentation separate was broken
Obsoletes:      %{go_name}-docs < 1.1-4

# RPM can't handle symlink -> dir with subpackages, so merge back
Obsoletes:      %{go_name}-data < 1.1.1-4

# emacs and vim subpackages no longer provided in 1.4.2
Obsoletes:      emacs-%{go_name} <= 1.3.3-3
Obsoletes:      %{go_name}-vim <= 1.3.3

# These are the only RHEL/Fedora architectures that we compile this package for
ExclusiveArch:  %{golang_arches}

Source100:      golang-gdbinit
Source101:      golang-prelink.conf
Source102:      macros.golang

%description
%{summary}.

%package       docs
Summary:       Golang compiler docs
Requires:      %{name} = %{version}-%{release}
BuildArch:     noarch
Obsoletes:     %{name}-docs < 1.1-4

%description   docs
%{summary}.

%package       misc
Summary:       Golang compiler miscellaneous sources
Requires:      %{name} = %{version}-%{release}
BuildArch:     noarch

%description   misc
%{summary}.

%package       tests
Summary:       Golang compiler tests for stdlib
Requires:      %{name} = %{version}-%{release}
BuildArch:     noarch

%description   tests
%{summary}.

%package        src
Summary:        Golang compiler source tree
BuildArch:      noarch
%description    src
%{summary}

%package        bin
Summary:        Golang core compiler tools
Requires:       go = %{version}-%{release}
# Pre-go1.5, all arches had to be bootstrapped individually, before usable, and
# env variables to compile for the target os-arch.
# Now the host compiler needs only the GOOS and GOARCH environment variables
# set to compile for the target os-arch.

# obsoletes *BSD sub packages
Obsoletes:  golang-pkg-plan9-386 <= 1.4.3
Obsoletes:  golang-pkg-plan9-amd64 <= 1.4.3
Obsoletes:  golang-pkg-freebsd-386 <= 1.4.3
Obsoletes:  golang-pkg-freebsd-amd64 <= 1.4.3
Obsoletes:  golang-pkg-freebsd-arm <= 1.4.3
Obsoletes:  golang-pkg-netbsd-386 <= 1.4.3
Obsoletes:  golang-pkg-netbsd-amd64 <= 1.4.3
Obsoletes:  golang-pkg-netbsd-arm <= 1.4.3
Obsoletes:  golang-pkg-openbsd-386 <= 1.4.3
Obsoletes:  golang-pkg-openbsd-amd64 <= 1.4.3
# obsolate go1.4 packages
Obsoletes:  golang-pkg-bin-linux-386 <= 1.4.3
Obsoletes:  golang-pkg-bin-linux-amd64 <= 1.4.3
Obsoletes:  golang-pkg-bin-linux-arm <= 1.4.3
Obsoletes:  golang-pkg-linux-386 <= 1.4.3
Obsoletes:  golang-pkg-linux-amd64 <= 1.4.3
Obsoletes:  golang-pkg-linux-arm <= 1.4.3
Obsoletes:  golang-pkg-darwin-386 <= 1.4.3
Obsoletes:  golang-pkg-darwin-amd64 <= 1.4.3
Obsoletes:  golang-pkg-windows-386 <= 1.4.3
Obsoletes:  golang-pkg-windows-amd64 <= 1.4.3

# We strip the meta dependency, but go does require glibc.
# This is an odd issue, still looking for a better fix.
Requires:       glibc
Requires:       gcc
%description    bin
%{summary}

# Workaround old RPM bug of symlink-replaced-with-dir failure
%pretrans -p <lua>
for _,d in pairs({"api", "doc", "include", "lib", "src"}) do
  path = "%{goroot}/" .. d
  if posix.stat(path, "type") == "link" then
    os.remove(path)
    posix.mkdir(path)
  end
end

%if %{shared}
%package        shared
Summary:        Golang shared object libraries

%description    shared
%{summary}.
%endif

%prep
%setup -q -n go

%patch220 -p1
%patch221 -p1

%build
# print out system information
uname -a
cat /proc/cpuinfo
cat /proc/meminfo

# bootstrap compiler GOROOT
%if !%{golang_bootstrap}
export GOROOT_BOOTSTRAP=/
%else
#export GOROOT_BOOTSTRAP=%{goroot}
# bootstrap should point to existing, non-armtoolset Golang
export GOROOT_BOOTSTRAP=/usr/lib/golang
%endif

# set up final install location
export GOROOT_FINAL=%{goroot}

export GOHOSTOS=linux
export GOHOSTARCH=%{gohostarch}

pushd src
# use our gcc options for this build, but store gcc as default for compiler
export CFLAGS="$RPM_OPT_FLAGS"
export LDFLAGS="$RPM_LD_FLAGS"
export CC="gcc"
export CC_FOR_TARGET="gcc"
export GOOS=linux
export GOARCH=%{gohostarch}
%if !%{external_linker}
export GO_LDFLAGS="-linkmode internal"
%endif
%if !%{cgo_enabled}
export CGO_ENABLED=0
%endif
./make.bash --no-clean
popd

# build shared std lib
%if %{shared}
GOROOT=$(pwd) PATH=$(pwd)/bin:$PATH go install -buildmode=shared std
%endif

%install
rm -rf $RPM_BUILD_ROOT

# create the top level directories
mkdir -p $RPM_BUILD_ROOT%{_bindir}
mkdir -p $RPM_BUILD_ROOT%{goroot}

# install everything into libdir (until symlink problems are fixed)
# https://code.google.com/p/go/issues/detail?id=5830
cp -apv api bin doc favicon.ico lib pkg robots.txt src misc test VERSION \
   $RPM_BUILD_ROOT%{goroot}

# bz1099206
find $RPM_BUILD_ROOT%{goroot}/src -exec touch -r $RPM_BUILD_ROOT%{goroot}/VERSION "{}" \;
# and level out all the built archives
touch $RPM_BUILD_ROOT%{goroot}/pkg
find $RPM_BUILD_ROOT%{goroot}/pkg -exec touch -r $RPM_BUILD_ROOT%{goroot}/pkg "{}" \;
# generate the spec file ownership of this source tree and packages
cwd=$(pwd)
src_list=$cwd/go-src.list
pkg_list=$cwd/go-pkg.list
shared_list=$cwd/go-shared.list
misc_list=$cwd/go-misc.list
docs_list=$cwd/go-docs.list
tests_list=$cwd/go-tests.list
rm -f $src_list $pkg_list $docs_list $misc_list $tests_list $shared_list
touch $src_list $pkg_list $docs_list $misc_list $tests_list $shared_list
pushd $RPM_BUILD_ROOT%{goroot}
    find src/ -type d -a \( ! -name testdata -a ! -ipath '*/testdata/*' \) -printf '%%%dir %{goroot}/%p\n' >> $src_list
    find src/ ! -type d -a \( ! -ipath '*/testdata/*' -a ! -name '*_test*.go' \) -printf '%{goroot}/%p\n' >> $src_list

    find bin/ pkg/ -type d -a ! -path '*_dynlink/*' -printf '%%%dir %{goroot}/%p\n' >> $pkg_list
    find bin/ pkg/ ! -type d -a ! -path '*_dynlink/*' -printf '%{goroot}/%p\n' >> $pkg_list

    find doc/ -type d -printf '%%%dir %{goroot}/%p\n' >> $docs_list
    find doc/ ! -type d -printf '%{goroot}/%p\n' >> $docs_list

    find misc/ -type d -printf '%%%dir %{goroot}/%p\n' >> $misc_list
    find misc/ ! -type d -printf '%{goroot}/%p\n' >> $misc_list

%if %{shared}
    find pkg/*_dynlink/ -type d -printf '%%%dir %{goroot}/%p\n' >> $shared_list
    find pkg/*_dynlink/ ! -type d -printf '%{goroot}/%p\n' >> $shared_list
%endif

    find test/ -type d -printf '%%%dir %{goroot}/%p\n' >> $tests_list
    find test/ ! -type d -printf '%{goroot}/%p\n' >> $tests_list
    find src/ -type d -a \( -name testdata -o -ipath '*/testdata/*' \) -printf '%%%dir %{goroot}/%p\n' >> $tests_list
    find src/ ! -type d -a \( -ipath '*/testdata/*' -o -name '*_test*.go' \) -printf '%{goroot}/%p\n' >> $tests_list
    # this is only the zoneinfo.zip
    find lib/ -type d -printf '%%%dir %{goroot}/%p\n' >> $tests_list
    find lib/ ! -type d -printf '%{goroot}/%p\n' >> $tests_list
popd

# remove the doc Makefile
rm -rfv $RPM_BUILD_ROOT%{goroot}/doc/Makefile

# put binaries to bindir, linked to the arch we're building,
# leave the arch independent pieces in {goroot}
mkdir -p $RPM_BUILD_ROOT%{goroot}/bin/linux_%{gohostarch}
ln -sf %{goroot}/bin/go $RPM_BUILD_ROOT%{goroot}/bin/linux_%{gohostarch}/go
ln -sf %{goroot}/bin/gofmt $RPM_BUILD_ROOT%{goroot}/bin/linux_%{gohostarch}/gofmt

# ensure these exist and are owned
mkdir -p $RPM_BUILD_ROOT%{gopath}/src/github.com
mkdir -p $RPM_BUILD_ROOT%{gopath}/src/bitbucket.org
mkdir -p $RPM_BUILD_ROOT%{gopath}/src/code.google.com/p
mkdir -p $RPM_BUILD_ROOT%{gopath}/src/golang.org/x

# make sure these files exist and point to alternatives
rm -f $RPM_BUILD_ROOT%{_bindir}/go
ln -sf %{_sysconfdir}/alternatives/go $RPM_BUILD_ROOT%{_bindir}/go
rm -f $RPM_BUILD_ROOT%{_bindir}/gofmt
ln -sf %{_sysconfdir}/alternatives/gofmt $RPM_BUILD_ROOT%{_bindir}/gofmt

# gdbinit
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit.d
cp -av %{SOURCE100} $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit.d/golang.gdb

# prelink blacklist
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/prelink.conf.d
cp -av %{SOURCE101} $RPM_BUILD_ROOT%{_sysconfdir}/prelink.conf.d/golang.conf

# rpm macros
mkdir -p %{buildroot}
%if 0%{?rhel} > 6 || 0%{?fedora} > 0
mkdir -p $RPM_BUILD_ROOT%{_rpmconfigdir2}/macros.d
cp -av %{SOURCE102} $RPM_BUILD_ROOT%{_rpmconfigdir2}/macros.d/macros.golang
%endif


%check
export GOROOT=$(pwd -P)
export PATH="$GOROOT"/bin:"$PATH"
cd src

export CC="gcc"
export CFLAGS="$RPM_OPT_FLAGS"
export LDFLAGS="$RPM_LD_FLAGS"
%if !%{external_linker}
export GO_LDFLAGS="-linkmode internal"
%endif
%if !%{cgo_enabled} || !%{external_linker}
export CGO_ENABLED=0
%endif
%if %{fail_on_tests}
./run.bash --no-rebuild -v -v -v -k
%else
./run.bash --no-rebuild -v -v -v -k || :
%endif
cd ..


%post bin
mkdir -p %{_sysconfdir}/alternatives
mkdir -p %{_scl_root}/var/lib/alternatives
%{alternatives_cmdline} --install %{_bindir}/go %{update_alt_pack_name} %{goroot}/bin/go 94 --slave %{_bindir}/gofmt gofmt %{goroot}/bin/gofmt

%preun bin
if [ $1 = 0 ]; then
    %{alternatives_cmdline} --remove %{update_alt_pack_name} %{goroot}/bin/go
fi


%files
%doc AUTHORS CONTRIBUTORS LICENSE PATENTS
# VERSION has to be present in the GOROOT, for `go install std` to work
%doc %{goroot}/VERSION
%dir %{goroot}/doc
%doc %{goroot}/doc/*

# go files
%dir %{goroot}
%exclude %{goroot}/bin/
%exclude %{goroot}/pkg/
%exclude %{goroot}/src/
%exclude %{goroot}/doc/
%exclude %{goroot}/misc/
%exclude %{goroot}/test/
%{goroot}/*

# ensure directory ownership, so they are cleaned up if empty
%dir %{gopath}
%dir %{gopath}/src
%dir %{gopath}/src/github.com/
%dir %{gopath}/src/bitbucket.org/
%dir %{gopath}/src/code.google.com/
%dir %{gopath}/src/code.google.com/p/
%dir %{gopath}/src/golang.org
%dir %{gopath}/src/golang.org/x


# gdbinit (for gdb debugging)
%{_sysconfdir}/gdbinit.d

# prelink blacklist
%{_sysconfdir}/prelink.conf.d

%if 0%{?rhel} > 6 || 0%{?fedora} > 0
%{_rpmconfigdir2}/macros.d/macros.golang
%endif

%files -f go-src.list src

%files -f go-docs.list docs

%files -f go-misc.list misc

%files -f go-tests.list tests

%files -f go-pkg.list bin
%ghost %{_bindir}/go
%ghost %{_bindir}/gofmt

%if %{shared}
%files -f go-shared.list shared
%endif

%changelog
* Mon Jan 07 2019 Jose E. Marchesi <jose.marchesi@oracle.com> - 1.10.3-2
- Temporarily disable the execution of testshared tests in aarch64.
- Fix a date in the .spec changelog.
- Reviewed-by: Elena Zannoni <elena.zannoni@oracle.com>

* Fri Jul 20 2018 Stephen Kyoungwon Kim <stephen.kim@oracle.com> - 1.10.3-1
- officially released go1.10.3 + the commit da769814b83c4eb65681dbe682d07d4b902b554a
  
* Wed Jul 04 2018 Stephen Kyoungwon Kim <stephen.kim@oracle.com> - 1.10.1-2
- Backport go1.10.3 patches from the master branch in the upstream
  da769814b83c4eb65681dbe682d07d4b902b554a: cmd/cgo: handle GCC 8 change in errors about constant initializers

* Tue May 15 2018 Stephen Kyoungwon Kim <stephen.kim@oracle.com> - 1.10.1-1
- Backport go1.10.2 patches from the release-go1.10 branch in the upstream
  5b6ff694bbc867f4ae95: cmd/compile: fix evaluation of "" < s
- Backport go1.10.2 patches from the release-go1.10 branch in the upstream
  abccc7a9b51372c67a0bc60159e8bf43ae77d102: misc/cgo/testshared seg fault with glibc-2.27
  
* Thu Apr 26 2018 Stephen Kyoungwon Kim <stephen.kim@oracle.com> - 1.10.1-0
- Started from the scratch based on Go1.10.1 for armtoolset-1
